
#include "stm32f10x_gpio.h"
#include "stm32f10x_i2c.h"
#include "bmp085.h"
#include "math.h"

const unsigned char OSS = 3;

void BMP085_Configuration(void)
{
	
	GPIO_InitTypeDef  GPIO_InitStructure; 
  I2C_InitTypeDef I2C_InitStructure;


  //GPIO Config
  GPIO_InitStructure.GPIO_Pin = BMP085_SCL | BMP085_SDA;      //I2C - SCL & SDA
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
  GPIO_Init(BMP085_PORT, &GPIO_InitStructure);
  
  //I2C Config
  I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;                //I2C mode
  I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;        //High to low state ratio of SCK, important for >100kHz transmission speed
  I2C_InitStructure.I2C_OwnAddress1 = 0x00;                 //I2C address at MCU side (if it works as slave)
  I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;               //Acknowledge enable
  I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit; //Addressing mode
  I2C_InitStructure.I2C_ClockSpeed=1000;                  //Clock frequency
  I2C_Init(BMP085_I2C,&I2C_InitStructure);

  I2C_Cmd(BMP085_I2C, ENABLE); 
	
}

//Odczyt danych z eeprom
void BMP085_Calibration(void)
{
		extern short ac1, ac2, ac3, b1, b2, mb, mc, md;
		extern unsigned short ac4, ac5, ac6;	
    ac1 = BMP085_ReadShort(0xAA);
    ac2 = BMP085_ReadShort(0xAC);
    ac3 = BMP085_ReadShort(0xAE);
    ac4 = BMP085_ReadShort(0xB0);
    ac5 = BMP085_ReadShort(0xB2);
    ac6 = BMP085_ReadShort(0xB4);
    b1 = BMP085_ReadShort(0xB6);
    b2 = BMP085_ReadShort(0xB8);
    mb = BMP085_ReadShort(0xBA);
    mc = BMP085_ReadShort(0xBC);
    md = BMP085_ReadShort(0xBE);
}


void BMP085_Convert(short* temperature, long* pressure, long* altitude)
{
	unsigned short ut;
  unsigned long up;
	float dp, wyk, alt;
	
  short x2, b5, b6, x3;
	long x1, p,b3;
  unsigned long b4;
	unsigned long b7;
	extern short ac1, ac2, ac3, b1, b2, mc, md;
	extern unsigned short ac4, ac5, ac6;
  //Odczyt tenperatury bezposrednio z czujnika
  ut = BMP085_ReadTemp();  
	//Odczyt cisnienia bezposrednio z czujnika	
  up = BMP085_ReadPressure();
	//Obliczanie temperatury   
  x1 = (((short)ut - (short)ac6) * (short)ac5) >> 15;
  x2 = ((short) mc << 11) / (x1 + md);
  b5 = x1 + x2;
  *temperature = (((b5 + 8) >> 4))/10;
	//Obliczanie cisnienia
  b6 = b5 - 4000;
  x1 = (b2 * (b6 * b6) >> 12) >> 11;
  x2 = (ac2 * b6) >> 11;
  x3 = x1 + x2;
	b3 = (((((short)ac1)*4 + x3)<<OSS) + 2)>>2;
	x1 = (ac3 * b6)>>13;
  x2 = (b1 * ((b6 * b6)>>12))>>16;
  x3 = ((x1 + x2) + 2)>>2;
  b4 = (ac4 * (unsigned long)(x3 + 32768))>>15;	
	b7 = ((unsigned long)(up - b3) * (50000>>OSS));
  if (b7 < 0x80000000)
    p = (b7<<1)/b4;
  else
    p = (b7/b4)<<1;	
	x1 = (p>>8) * (p>>8);
  x1 = (x1 * 3038)>>16;
  x2 = (-7357 * p)>>16;
  p += (x1 + x2 + 3791)>>4;	
  *pressure = p;
	//Obliczanie wysokosci
	dp=((float)p)/101325;
	wyk=1/5.25588;
	alt=1-(pow(dp,wyk));
	alt=alt*4433000;//0.0000225577;
	*altitude=(long)alt;
	
}

unsigned short BMP085_ReadTemp(void)
{
	
		BMP085_WriteRegister(0xF4, 0x2E);
    delay_ms(50);    // max time is 4.5ms
    return BMP085_ReadShort(0xF6);
}

unsigned long BMP085_ReadPressure(void)
{
		unsigned char msb, lsb, xlsb;	
	
		BMP085_WriteRegister(0xF4, (0x34+(OSS<<6)));
    delay_ms(50);    // max time is 4.5ms
	
	
	I2C_GenerateSTART(BMP085_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_MODE_SELECT));  
  //Send LIS35 address, set I2C master in transmiter mode
  I2C_Send7bitAddress(BMP085_I2C, BMP085_Addr, I2C_Direction_Transmitter);
  //Test on EV6 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  //Send base register address, set address autoincrement
  I2C_SendData(BMP085_I2C, 0xF6);
  //Test on EV8 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
	
  //Re-generate START, transmition from slave beginning
  I2C_GenerateSTART(BMP085_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_MODE_SELECT)); 
  //Send LIS35 address, set I2C master in receiver mode
  I2C_Send7bitAddress(BMP085_I2C, BMP085_Addr, I2C_Direction_Receiver);
  //Test on EV6 and clear it 
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));  
	
  //Test on EV7 and clear it
	while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));
  //Read a byte from the LI35 
  msb = I2C_ReceiveData(BMP085_I2C);
	//Test on EV7 and clear it
	while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));
  //Read a byte from the LI35 
  lsb = I2C_ReceiveData(BMP085_I2C);
	
	//Before receiving last byte, disable acknowledge and generate stop
	I2C_AcknowledgeConfig(BMP085_I2C, DISABLE);
  I2C_GenerateSTOP(BMP085_I2C, ENABLE);
	//Test on EV7 and clear it
	while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));
  //Read a byte from the LI35 
  xlsb = I2C_ReceiveData(BMP085_I2C);
	//Enable Acknowledge for next transmission
	I2C_AcknowledgeConfig(BMP085_I2C, ENABLE);
   
	return (((unsigned short) msb << 16) | ((unsigned short) lsb << 8) | ((unsigned short) xlsb)) >> (8-OSS);
}


void BMP085_WriteRegister(char addr,char v)
{
  //Writes one register value 

  I2C_GenerateSTART(BMP085_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_MODE_SELECT));  
  //Send LIS35 address, set I2C master in transmiter mode
  I2C_Send7bitAddress(BMP085_I2C, BMP085_Addr, I2C_Direction_Transmitter);
  //Test on EV6 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  //Send LIS35 local register address, which will be written 
  I2C_SendData(BMP085_I2C, addr);
  //Test on EV8 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  //Send new register value
  I2C_SendData(BMP085_I2C, v);
  //Test on EV8 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  //Generate STOP Condition
  I2C_GenerateSTOP(BMP085_I2C,ENABLE); 
}


// bmp085ReadShort will read two sequential 8-bit registers, and return a 16-bit value
// the MSB register is read first
// Input: First register to read
// Output: 16-bit value of (first register value << 8) | (sequential register value)

short BMP085_ReadShort(unsigned char address)
{
		unsigned char msb, lsb;
    short data;
   	
  I2C_GenerateSTART(BMP085_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_MODE_SELECT));  
  //Send LIS35 address, set I2C master in transmiter mode
  I2C_Send7bitAddress(BMP085_I2C, BMP085_Addr, I2C_Direction_Transmitter);
  //Test on EV6 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
  //Send base register address, set address autoincrement
  I2C_SendData(BMP085_I2C, address);
  //Test on EV8 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
  //Re-generate START, transmition from slave beginning
  I2C_GenerateSTART(BMP085_I2C,ENABLE);
  //Test on EV5 and clear it
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_MODE_SELECT)); 
  //Send LIS35 address, set I2C master in receiver mode
  I2C_Send7bitAddress(BMP085_I2C, BMP085_Addr, I2C_Direction_Receiver);
  //Test on EV6 and clear it 
  while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED));   
  //Test on EV7 and clear it
	while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));
  //Read a byte from the LI35 
  msb = I2C_ReceiveData(BMP085_I2C);
	//Before receiving last byte, disable acknowledge and generate stop
	I2C_AcknowledgeConfig(BMP085_I2C, DISABLE);
  I2C_GenerateSTOP(BMP085_I2C, ENABLE);
	//Test on EV7 and clear it
	while(!I2C_CheckEvent(BMP085_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED));
  //Read a byte from the LI35 
  lsb = I2C_ReceiveData(BMP085_I2C);
	//Enable Acknowledge for next transmission
	I2C_AcknowledgeConfig(BMP085_I2C, ENABLE);
   
    data = msb << 8;
    data |= lsb;
    return data;
}


void delay_ms(char ilosc)
{
	long i;
	
	if(ilosc!=0)
	{
		for(i=0;i<100000;i++)
		ilosc--;
	}
}
	
	
